use Terminal::ANSIColor;

unit module Grammar::Message;

sub pretty-message( Str $msg, Match $match ) is export {
    my @msg = ["❗ " ~ color-word("blue",$msg)];
    my @lines = $match.target.lines;
    my @lines-ranges;
    my $initial-char = 0;
    my $line-no = 0;
    for @lines -> $l {
        my $final-char = $initial-char + -1 + $l.chars;
        @lines-ranges.push:
                $l => $initial-char .. $final-char;
        $line-no = @lines-ranges.elems - 1
            if $initial-char <= $match.pos <= $final-char;
        $initial-char = $final-char + 1;
    }
    my $first = ( ($line-no - 3) max 0 );
    my @near = @lines[ $first.. (($line-no + 3) min @lines-1) ];
    my $i = $line-no - 3 max 0;

    for @near {
        $i++;
        if $i==$line-no {
            my $column = $match.pos - @lines-ranges[$i].value.min;
            @msg.push: color-word('bold red', $i.fmt("%3d") ~ " │▶ $_");
            @msg.push: "     " ~ '^'.indent($column);
        } else {
            @msg.push: color-word('magenta', $i.fmt("%3d") ~ " │ ") ~ $_;
        }
    }
    @msg.push: "";
    return @msg.join("\n");
}

sub color-word( $color, $word ) {
    return color($color) ~ $word ~ color('reset');
}
